/***
 * @Copyright 2022-2023 SPACEMIT. All rights reserved.
 * @Use of this source code is governed by a BSD-style license
 * @that can be found in the LICENSE file.
 * @
 * @Author: David(qiang.fu@spacemit.com)
 * @Date: 2023-08-30 15:31:42
 * @LastEditTime: 2023-09-20 14:03:44
 * @Description:
 */

#ifndef LIBV4L_STMPP_DEC_H
#define LIBV4L_STMPP_DEC_H

#include "libv4l-stmpp.h"

/**
 * struct stmpp_video_info - Video information
 * @valid:		Data is valid.
 * @dirty:		Data is dirty(have not applied to mpp).
 * @event:		Pending V4L2 src_change event.
 * @width:		Video width.
 * @height:		Video height.
 * @hor_stride:		Video horizontal stride.
 * @ver_stride:		Video vertical stride.
 * @size:		Required video buffer size.
 */
struct stmpp_video_info {
  bool valid;
  bool dirty;
  bool event;

  uint32_t width;
  uint32_t height;
  uint32_t hor_stride;
  uint32_t ver_stride;

  uint32_t size;
};

/**
 * struct stmpp_dec_context - Context private data for decoder
 * @ctx:		Common context data.
 * @video_info:		Video information.
 * @event_subscribed:	V4L2 event subscribed.
 * @mpp_streaming:	The mpp is streaming.
 * @decoder_thread:	Handler of the decoder thread.
 * @decoder_cond:	Condition variable for streaming flag.
 * @decoder_mutex:	Mutex for streaming flag and buffers.
 */
struct stmpp_dec_context {
  struct stmpp_context *ctx;
  struct stmpp_video_info video_info;

  MppVdecCtx *pVdecCtx;

  bool event_subscribed;

  bool mpp_streaming;

  struct stmpp_buffer *eos_packet;

  pthread_t decoder_thread;
  pthread_cond_t decoder_cond;
  pthread_mutex_t decoder_mutex;
};

bool stmpp_dec_has_event(void *data);
void *stmpp_dec_init(struct stmpp_context *ctx);
int stmpp_dec_ioctl(void *data, unsigned long cmd, void *arg);
void stmpp_dec_deinit(void *data);

#endif  // LIBV4L_STMPP_DEC_H
